"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.computeBlockMap = exports.createPackageFileInfo = exports.createDifferentialPackage = undefined;

var _bluebirdLst;

function _load_bluebirdLst() {
    return _bluebirdLst = require("bluebird-lst");
}

var _bluebirdLst2;

function _load_bluebirdLst2() {
    return _bluebirdLst2 = _interopRequireDefault(require("bluebird-lst"));
}

/*
Approach like AppX block map, but with one difference - block not compressed individually, instead, the whole file is compressed using LZMA compression.
See (Package File in the developer readme) about compression. So, delta will be not ideal (because compressed data can change not only actually changed block in the file, but others,
and we don't set even dict size and default 64M is used), but full package size will be still relative small and will save initial download time/costs.
 */
// reduce dict size to avoid large block invalidation on change
let createDifferentialPackage = exports.createDifferentialPackage = (() => {
    var _ref = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (archiveFile, packager) {
        // Deflate to be able to decompress block map on the fly without 7za
        const args = (0, (_archive || _load_archive()).compute7zCompressArgs)("normal", "zip", { method: "Deflate" });
        args.push(archiveFile);
        // compute block map using compressed file data
        const blockMap = yield computeBlockMap(archiveFile, "lzma", process.env.ELECTRON_BUILDER_COMPRESSION_LEVEL || "9");
        // https://superuser.com/a/966241, use leading ~ symbol to add file to the end (and avoid file rewrite)
        const blockMapFile = yield packager.info.tempDirManager.getTempFile({ prefix: "~", suffix: "blockMap.yml" });
        yield (0, (_fsExtraP || _load_fsExtraP()).outputFile)(blockMapFile, (0, (_jsYaml || _load_jsYaml()).safeDump)(blockMap));
        yield (0, (_builderUtil || _load_builderUtil()).spawn)((_zipBin || _load_zipBin()).path7za, args.concat(blockMapFile), {
            cwd: _path.dirname(archiveFile)
        });
        // rename to final name (file order will be not changed and preserved as is)
        yield (0, (_builderUtil || _load_builderUtil()).exec)((_zipBin || _load_zipBin()).path7za, ["rn", archiveFile, _path.basename(blockMapFile), (_blockMapApi || _load_blockMapApi()).BLOCK_MAP_FILE_NAME]);
        return yield createPackageFileInfo(archiveFile);
    });

    return function createDifferentialPackage(_x, _x2) {
        return _ref.apply(this, arguments);
    };
})();

let createPackageFileInfo = exports.createPackageFileInfo = (() => {
    var _ref2 = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (file) {
        return {
            file,
            size: (yield (0, (_fsExtraP || _load_fsExtraP()).stat)(file)).size,
            sha512: yield (0, (_builderUtil || _load_builderUtil()).hashFile)(file)
        };
    });

    return function createPackageFileInfo(_x3) {
        return _ref2.apply(this, arguments);
    };
})();

let computeBlocks = (() => {
    var _ref3 = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (fd, start, end) {
        const chunkSize = 64 * 1024;
        const buffer = Buffer.allocUnsafe(chunkSize);
        const blocks = [];
        for (let offset = start; offset < end; offset += chunkSize) {
            const actualChunkSize = Math.min(end - offset, chunkSize);
            yield (0, (_fsExtraP || _load_fsExtraP()).read)(fd, buffer, 0, actualChunkSize, offset);
            const hash = (0, (_crypto || _load_crypto()).createHash)("md5");
            hash.update(actualChunkSize === chunkSize ? buffer : buffer.slice(0, actualChunkSize));
            blocks.push(hash.digest("base64"));
        }
        return blocks;
    });

    return function computeBlocks(_x4, _x5, _x6) {
        return _ref3.apply(this, arguments);
    };
})();

let computeBlockMap = exports.computeBlockMap = (() => {
    var _ref4 = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (archiveFile, compressionMethod, compressionLevel) {
        const zip = yield (0, (_localZip || _load_localZip()).openZip)(archiveFile);
        try {
            const entries = yield zip.readEntries();
            const files = yield (_bluebirdLst2 || _load_bluebirdLst2()).default.map(entries, (() => {
                var _ref5 = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (entry) {
                    const blocks = yield computeBlocks(zip.fd, entry.dataStart, entry.dataStart + entry.compressedSize);
                    return {
                        name: entry.fileName.replace(/\\/g, "/"),
                        size: entry.compressedSize,
                        blocks
                    };
                });

                return function (_x10) {
                    return _ref5.apply(this, arguments);
                };
            })(), { concurrency: 8 });
            return {
                blockSize: 64,
                hashMethod: "md5",
                compressionMethod,
                compressionLevel,
                files
            };
        } finally {
            yield zip.close();
        }
    });

    return function computeBlockMap(_x7, _x8, _x9) {
        return _ref4.apply(this, arguments);
    };
})();
//# sourceMappingURL=blockMap.js.map


var _zipBin;

function _load_zipBin() {
    return _zipBin = require("7zip-bin");
}

var _builderUtil;

function _load_builderUtil() {
    return _builderUtil = require("builder-util");
}

var _blockMapApi;

function _load_blockMapApi() {
    return _blockMapApi = require("builder-util-runtime/out/differentialUpdate/blockMapApi");
}

var _localZip;

function _load_localZip() {
    return _localZip = require("builder-util-runtime/out/differentialUpdate/localZip");
}

var _crypto;

function _load_crypto() {
    return _crypto = require("crypto");
}

var _fsExtraP;

function _load_fsExtraP() {
    return _fsExtraP = require("fs-extra-p");
}

var _jsYaml;

function _load_jsYaml() {
    return _jsYaml = require("js-yaml");
}

var _path = _interopRequireWildcard(require("path"));

var _archive;

function _load_archive() {
    return _archive = require("./archive");
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }